/**
 * Easy soundcloud downloader
 * ==========================
 *
 * Adds a button to all the songs on a soundcloud webpage & also on widgets!
 * No more copy pasting!
 *
 * ToDo: On other website with the SC widget, on user profile
 */

// Global vars
var logo;
var hasLogo = false;
var isReady = false;


/**
 * @brief Event listener
 * @details Listens for events from the plugin
 */
window.addEventListener("scdLogo", function(evt)
{
	// get the new logo location
	logo = evt.detail;


	if(isReady)
		updatePage();

}, false);


/**
 * @brief Called when the document has loaded
*/
$(document).ready(function()
{
	MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
/*
	var observer = new MutationObserver(function(mutations) {
	    // fired when a mutation occurs
	    updatePage();
	    console.log(mutations);
	});

	// define what element should be observed by the observer
	// and what types of mutations trigger the callback
	observer.observe(document.querySelector("#content"), {
	  childList: true
	});*/


	if(hasLogo)
		// Update the page now
		updatePage();

	var hashDetection = new hashHandler();

    isReady = true;
});

/**
 * @brief Sort of a has handler
 * @details Will change changes in URL
 * @return [description]
 */
function hashHandler(){
    this.oldHash = window.location.href;
    this.Check;

    var that = this;
    var detect = function(){
        if(that.oldHash!=window.location.href){
            
        	$("#content div").ready(function(){
        		updatePage();
        	});

            that.oldHash = window.location.href;
        }
    };
    this.Check = setInterval(function(){ detect() }, 1000);
}



/**
 * @brief Will update the page with the download icons
 * @details Adds all the download buttons to the avaliable songs
 */
function updatePage()
{
	console.log("load");

	// Get the current location
	var loc = window.location.pathname;

	// Remove all
	$(".scdbtn").each(function(){
		$(this).detach();
	});


	// Check on which page we are
	if(loc == "/stream")
	{
		// Stream page
		stream();
	}
	else if(loc.match(/^\/.+\/sets\/.+\/?$/))
	{
		aSet();
	}
	else if(loc.match(/^\/.+\/.+\/sets\/?$/))
	{
		// sets page
		sets();
	}
	else if(loc.match(/\/.+\/.+\/?/))
	{
		// Sound page
		sound();
	}
}



/**
 * @brief Creates all the buttons on the stream page
 * @details Will add all the buttons to the stream page on SC
 */
function stream()
{
	$( ".soundList__item" ).each(function( index )
	{
		var url = $(this).find(".soundTitle__title").attr("href");

		$(this).find(".soundActions .sc-button-group:first-child").append(button(url, "small"));
	});

	var obs = new MutationObserver(function(mu)
	{
		updatePage();
	});

	if ($(".lazyLoadingList__list")[0]){
    		// Do something if class exists
    		obs.observe(document.querySelector(".lazyLoadingList__list"), {childList: true});
	}
}

/**
 * @brief Creates all the buttons on the song page
 * @details Will add all the buttons on the songs page on SC
 */
function sound()
{
	var url = window.location.href;

	$(".soundActions .sc-button-group:first-child").append(button(url, "medium"));
}

/**
 * @brief Creates all the buttons on the sets page
 * @details Will add all the buttons to the sets page on SC
 */
function sets()
{
	$( ".soundList__item" ).each(function( index )
	{
		var url = $(this).find(".soundTitle__title").attr("href");

		$(this).find(".soundActions .sc-button-group:first-child").append(button(url, "small"));
	});
}

/**
 * @brief Adds the download button to a single playlist
 */
function aSet()
{
	var url = window.location.href;

	$(".listenEngagement__actions .soundActions .sc-button-group:first-child").append(button(url, "medium"));


	// Now let's do the songs
	$( ".listenDetails .trackList__item" ).each(function( index )
	{
		var url = $(this).find(".trackItem__trackTitle").attr("href");

		$(this).find(".soundActions .sc-button-group:first-child").append(button(url, "small"));
	});




	var obs = new MutationObserver(function(muu)
	{
		updatePage();
	});

	obs.observe(document.querySelector(".trackList__list"),{childList: true});
}





/**
 * @brief 		Create a SCD url
 * @details 	Creates a URL for SCD to download the url/song
 * 
 * @param  	The SC URL
 * @return 		URL to open SCD with
 */	
function scdURL(url)
{
	// Prepend some text
	return "scd://download?link=https://soundcloud.com" + url;
}

/**
 * @brief 			SCD download button
 * @details 		Creates a matching button for soundcloud to download the song with SCD
 * 
 * @param url 		URL to the song
 * @param size 	The size of the button (small, medium, large)
 * 
 * @return 			A full & matching HTML button
 */
function button(url, size)
{
	// get the SCD URL
	var newURL = scdURL(url);

	return "<a href='" + newURL + "' class='sc-button sc-button sc-button-" + size + " sc-button-responsive scdbtn' tabindex='0' title='Download with SCD'>"+
				"<img src='" + logo + "' />"+
			"</a>";
}